@@ -21,7 +21,7 @@ has defaultValuesFile     => ( isa => 'Str', is => 'ro', required => 1 );
 has confConstantsFile     => ( isa => 'Str', is => 'ro', required => 1 );
 has firstLmConfFile       => ( isa => 'Str', is => 'ro', required => 1 );
 
-my @managerAttrKeys = qw(keyTest select type test msgFail default);
+my @managerAttrKeys = qw(keyTest keyMsgFail select type test msgFail default);
 my $format          = 'Creating %-69s: ';
 my $reIgnoreKeys    = qr/^$/;
 my $module          = __PACKAGE__;
