@@ -6,7 +6,7 @@
 
 package Lemonldap::NG::Manager::Build::Attributes;
 
-our $VERSION = '1.9.0';
+our $VERSION = '1.9.1';
 use strict;
 use Regexp::Common qw/URI/;
 
@@ -72,14 +72,14 @@ sub types {
             form => 'text',
             test => sub {
                 my ( $val, $conf ) = @_;
-                if (   defined $conf->{exportedVars}->{$val}
-                    or defined $conf->{macros}->{$val} )
-                {
-                    return 1;
-                }
-                else {
-                    return ( 1, "__unknownAttrOrMacro__: $val" );
+                return 1
+                  if ( defined $conf->{macros}->{$val} or $val eq '_timezone' );
+                foreach ( keys %$conf ) {
+                    return 1
+                      if ( $_ =~ /exportedvars$/i
+                        and defined $conf->{$_}->{$val} );
                 }
+                return ( 1, "__unknownAttrOrMacro__: $val" );
             },
         },
 
@@ -102,9 +102,9 @@ sub types {
         },
         keyTextContainer => {
             test       => qr/./,
-            msgFail    => 'Empty value non accorded',
+            msgFail    => '__emptyValueNotAllowed__',
             keyTest    => qr/^\w[\w\.\-]*$/,
-            keyMsgFail => 'Bad key name',
+            keyMsgFail => '__badKeyName__',
         },
         subContainer => {
             keyTest => qr/\w/,
@@ -126,19 +126,34 @@ sub types {
             test => sub { 1 }
         },
         RSAPublicKey => {
-            test =>
-qr/^(?:(?:\-+\s*BEGIN\s+PUBLIC\s+KEY\s*\-+\r?\n)?[a-zA-Z0-9\/\+\r\n]+={0,2}(?:\r?\n\-+\s*END\s+PUBLIC\s+KEY\s*\-+)?[\r\n]*)?$/s,
-            msgFail => '__badPemEncoding__',
+            test => sub {
+                return (
+                    $_[0] =~
+/^(?:(?:\-+\s*BEGIN\s+PUBLIC\s+KEY\s*\-+\r?\n)?[a-zA-Z0-9\/\+\r\n]+={0,2}(?:\r?\n\-+\s*END\s+PUBLIC\s+KEY\s*\-+)?[\r\n]*)?$/s
+                    ? (1)
+                    : ( 1, '__badPemEncoding__' )
+                );
+            },
         },
         'RSAPublicKeyOrCertificate' => {
-            'test' =>
-qr/^(?:(?:\-+\s*BEGIN\s+(?:PUBLIC\s+KEY|CERTIFICATE)\s*\-+\r?\n)?[a-zA-Z0-9\/\+\r\n]+={0,2}(?:\r?\n\-+\s*END\s+(?:PUBLIC\s+KEY|CERTIFICATE)\s*\-+)?[\r\n]*)?$/s,
-            'msgFail' => '__badPemEncoding__',
+            'test' => sub {
+                return (
+                    $_[0] =~
+/^(?:(?:\-+\s*BEGIN\s+(?:PUBLIC\s+KEY|CERTIFICATE)\s*\-+\r?\n)?[a-zA-Z0-9\/\+\r\n]+={0,2}(?:\r?\n\-+\s*END\s+(?:PUBLIC\s+KEY|CERTIFICATE)\s*\-+)?[\r\n]*)?$/s
+                    ? (1)
+                    : ( 1, '__badPemEncoding__' )
+                );
+            },
         },
         RSAPrivateKey => {
-            test =>
-qr/^(?:(?:\-+\s*BEGIN\s+RSA\s+PRIVATE\s+KEY\s*\-+\r?\n)?[a-zA-Z0-9\/\+\r\n]+={0,2}(?:\r?\n\-+\s*END\s+RSA\s+PRIVATE\s+KEY\s*\-+)?[\r\n]*)?$/s,
-            msgFail => '__badPemEncoding__',
+            test => sub {
+                return (
+                    $_[0] =~
+/^(?:(?:\-+\s*BEGIN\s+(?:RSA\s+)?PRIVATE\s+KEY\s*\-+\r?\n)?[a-zA-Z0-9\/\+\r\n]+={0,2}(?:\r?\n\-+\s*END\s+(?:RSA\s+)PRIVATE\s+KEY\s*\-+)?[\r\n]*)?$/s
+                    ? (1)
+                    : ( 1, '__badPemEncoding__' )
+                );
+            },
         },
 
         authParamsText => {
@@ -389,7 +404,7 @@ sub attributes {
             type       => 'keyTextContainer',
             help       => 'portalcustom.html',
             keyTest    => $perlExpr,
-            keyMsgFail => 'Bad skin rule',
+            keyMsgFail => '__badSkinRule__',
             test       => qr/^\w+$/,
             msgFail    => '__badValue__',
         },
@@ -640,7 +655,7 @@ sub attributes {
             type          => 'keyTextContainer',
             help          => 'exportedvars.html',
             keyTest       => qr/^!?[_a-zA-Z][a-zA-Z0-9_]*$/,
-            keyMsgFail    => 'Bad variable name',
+            keyMsgFail    => '__badVariableName__',
             test          => qr/^[_a-zA-Z][a-zA-Z0-9_:\-]*$/,
             msgFail       => '__badValue__',
             default       => { 'UA' => 'HTTP_USER_AGENT' },
@@ -659,7 +674,7 @@ sub attributes {
             help =>
               'exportedvars.html#extend_variables_using_macros_and_groups',
             keyTest       => qr/^[_a-zA-Z][a-zA-Z0-9_]*$/,
-            keyMsgFail    => 'Bad macro name',
+            keyMsgFail    => '__badMacroName__',
             test          => $perlExpr,
             default       => {},
             documentation => 'Macros',
@@ -704,7 +719,7 @@ sub attributes {
         sessionDataToRemember    => {
             type       => 'keyTextContainer',
             keyTest    => qr/^[_a-zA-Z][a-zA-Z0-9_]*$/,
-            keyMsgFail => 'Invalid session data',
+            keyMsgFail => '__invalidSessionData__',
         },
 
         # SAML issuer
@@ -904,7 +919,7 @@ sub attributes {
                           ? (1)
                           : ( 0, '__badUrl__' );
                     }
-                    $s =~ s/\b(accept|deny)\b/1/g;
+                    $s =~ s/\b(accept|deny|unprotect|skip)\b/1/g;
                     my @cf =
                       qw(encode_base64 checkLogonHours date checkDate basic unicode2iso iso2unicode groupMatch);
                     push @cf,
@@ -928,10 +943,30 @@ sub attributes {
             documentation => 'Virtualhost rules',
         },
         exportedHeaders => {
-            type          => 'keyTextContainer',
-            help          => 'writingrulesand_headers.html#headers',
-            keyTest       => qr/^$Regexp::Common::URI::RFC2396::hostname$/,
-            keyMsgFail    => '__badHostname__',
+            type       => 'keyTextContainer',
+            help       => 'writingrulesand_headers.html#headers',
+            keyTest    => qr/^$Regexp::Common::URI::RFC2396::hostname$/,
+            keyMsgFail => '__badHostname__',
+            test       => {
+                keyTest    => qr/^(?=[^\-])[\w\-]+(?<=[^-])$/,
+                keyMsgFail => '__badHeaderName__',
+                test       => sub {
+                    my ( $val, $conf ) = @_;
+                    my $s = $val;
+                    my @cf =
+                      qw(encode_base64 checkLogonHours date checkDate basic unicode2iso iso2unicode groupMatch);
+                    push @cf,
+                      defined $conf->{customFunctions}
+                      ? map { my $f = $_; $f =~ s/\w+:://g; ( $f, $_ ) }
+                      split( /\s+/, $conf->{customFunctions} )
+                      : ();
+                    foreach my $f (@cf) {
+                        $s = "sub $f {1} $s";
+                    }
+                    eval $s;
+                    return $@ ? ( 0, "__badExpression__: $@" ) : (1);
+                  }
+            },
             documentation => 'Virtualhost headers',
         },
         post => {
@@ -1314,7 +1349,7 @@ sub attributes {
             type       => 'samlAttributeContainer',
             help       => 'authsaml.html#exported_attributes',
             keyTest    => qr/^[a-zA-Z](?:[a-zA-Z0-9_\-\.]*\w)?$/,
-            keyMsgFail => 'Bad metadata name',
+            keyMsgFail => '__badMetadataName__',
             test       => qr/\w/,
             msgFail    => '__badValue__',
             default    => {},
@@ -1325,7 +1360,7 @@ sub attributes {
         samlIDPMetaDataOptions => {
             type       => 'keyTextContainer',
             keyTest    => qr/^[a-zA-Z](?:[a-zA-Z0-9_\-\.]*\w)?$/,
-            keyMsgFail => 'Bad metadata name',
+            keyMsgFail => '__badMetadataName__',
         },
         samlIDPMetaDataOptionsNameIDFormat => {
             type   => 'select',
@@ -1431,7 +1466,7 @@ sub attributes {
             type       => 'samlAttributeContainer',
             help       => 'idpsaml.html#exported_attributes',
             keyTest    => qr/^[a-zA-Z](?:[a-zA-Z0-9_\-\.]*\w)?$/,
-            keyMsgFail => 'Bad metadata name',
+            keyMsgFail => '__badMetadataName__',
             test       => qr/\w/,
             msgFail    => '__badValue__',
             default    => {},
@@ -1442,7 +1477,7 @@ sub attributes {
         samlSPMetaDataOptions => {
             type       => 'keyTextContainer',
             keyTest    => qr/^[a-zA-Z](?:[a-zA-Z0-9_\-\.]*\w)?$/,
-            keyMsgFail => 'Bad metadata name',
+            keyMsgFail => '__badMetadataName__',
         },
         samlSPSSODescriptorAuthnRequestsSigned => {
             default       => 1,
@@ -1626,7 +1661,7 @@ sub attributes {
         demoExportedVars => {
             type          => 'keyTextContainer',
             keyTest       => qr/^!?[a-zA-Z][a-zA-Z0-9_-]*$/,
-            keyMsgFail    => 'Bad variable name',
+            keyMsgFail    => '__badVariableName__',
             test          => qr/^[a-zA-Z][a-zA-Z0-9_:\-]*$/,
             msgFail       => '__badValue__',
             default       => { cn => 'cn', mail => 'mail', uid => 'uid', },
@@ -1675,7 +1710,7 @@ sub attributes {
         ldapExportedVars => {
             type          => 'keyTextContainer',
             keyTest       => qr/^!?[a-zA-Z][a-zA-Z0-9_-]*$/,
-            keyMsgFail    => 'Bad variable name',
+            keyMsgFail    => '__badVariableName__',
             test          => qr/^[a-zA-Z][a-zA-Z0-9_:\-]*$/,
             msgFail       => '__badValue__',
             default       => { cn => 'cn', mail => 'mail', uid => 'uid', },
@@ -1694,7 +1729,7 @@ sub attributes {
                 foreach my $s (@s) {
                     $s =~
 m{^(?:ldapi://[^/]*/?|\w[\w\-\.]*(?::\d{1,5})?|ldap(?:s|\+tls)?://\w[\w\-\.]*(?::\d{1,5})?/?.*)$}o
-                      or return ( 0, "Bad ldap uri \"$s\"" );
+                      or return ( 0, "__badLdapUri__: \"$s\"" );
                 }
                 return 1;
             },
@@ -1825,7 +1860,7 @@ m{^(?:ldapi://[^/]*/?|\w[\w\-\.]*(?::\d{1,5})?|ldap(?:s|\+tls)?://\w[\w\-\.]*(?:
         CAS_proxiedServices => {
             type       => 'keyTextContainer',
             keyTest    => qr/^\w+$/,
-            keyMsgFail => 'Bad CAS proxied service identifier',
+            keyMsgFail => '__badCasProxyId__',
         },
 
         # Radius
@@ -1869,7 +1904,7 @@ m{^(?:ldapi://[^/]*/?|\w[\w\-\.]*(?::\d{1,5})?|ldap(?:s|\+tls)?://\w[\w\-\.]*(?:
         openIdExportedVars => {
             type          => 'keyTextContainer',
             keyTest       => qr/^!?[a-zA-Z][a-zA-Z0-9_-]*$/,
-            keyMsgFail    => 'Bad variable name',
+            keyMsgFail    => '__badVariableName__',
             test          => qr/^[a-zA-Z][a-zA-Z0-9_:\-]*$/,
             msgFail       => '__badValue__',
             default       => {},
@@ -1886,7 +1921,7 @@ m{^(?:ldapi://[^/]*/?|\w[\w\-\.]*(?::\d{1,5})?|ldap(?:s|\+tls)?://\w[\w\-\.]*(?:
         googleExportedVars => {
             type          => 'keyTextContainer',
             keyTest       => qr/^!?[a-zA-Z][a-zA-Z0-9_-]*$/,
-            keyMsgFail    => 'Bad variable name',
+            keyMsgFail    => '__badVariableName__',
             test          => qr/^[a-zA-Z][a-zA-Z0-9_:\-]*$/,
             msgFail       => '__badValue__',
             default       => {},
@@ -1904,7 +1939,7 @@ m{^(?:ldapi://[^/]*/?|\w[\w\-\.]*(?::\d{1,5})?|ldap(?:s|\+tls)?://\w[\w\-\.]*(?:
         facebookExportedVars => {
             type          => 'keyTextContainer',
             keyTest       => qr/^!?[a-zA-Z][a-zA-Z0-9_-]*$/,
-            keyMsgFail    => 'Bad variable name',
+            keyMsgFail    => '__badVariableName__',
             test          => qr/^[a-zA-Z][a-zA-Z0-9_:\-]*$/,
             msgFail       => '__badValue__',
             default       => {},
@@ -1931,7 +1966,7 @@ m{^(?:ldapi://[^/]*/?|\w[\w\-\.]*(?::\d{1,5})?|ldap(?:s|\+tls)?://\w[\w\-\.]*(?:
         webIDExportedVars => {
             type          => 'keyTextContainer',
             keyTest       => qr/^!?[a-zA-Z][a-zA-Z0-9_-]*$/,
-            keyMsgFail    => 'Bad variable name',
+            keyMsgFail    => '__badVariableName__',
             test          => qr/^[a-zA-Z][a-zA-Z0-9_:\-]*$/,
             msgFail       => '__badValue__',
             default       => {},
@@ -1961,7 +1996,7 @@ m{^(?:ldapi://[^/]*/?|\w[\w\-\.]*(?::\d{1,5})?|ldap(?:s|\+tls)?://\w[\w\-\.]*(?:
         dbiExportedVars => {
             type          => 'keyTextContainer',
             keyTest       => qr/^!?[a-zA-Z][a-zA-Z0-9_-]*$/,
-            keyMsgFail    => 'Bad variable name',
+            keyMsgFail    => '__badVariableName__',
             test          => qr/^[a-zA-Z][a-zA-Z0-9_:\-]*$/,
             msgFail       => '__badValue__',
             default       => {},
@@ -1992,7 +2027,7 @@ m{^(?:ldapi://[^/]*/?|\w[\w\-\.]*(?::\d{1,5})?|ldap(?:s|\+tls)?://\w[\w\-\.]*(?:
         slaveExportedVars => {
             type          => 'keyTextContainer',
             keyTest       => qr/^!?[a-zA-Z][a-zA-Z0-9_-]*$/,
-            keyMsgFail    => 'Bad variable name',
+            keyMsgFail    => '__badVariableName__',
             test          => qr/^[a-zA-Z][a-zA-Z0-9_:\-]*$/,
             msgFail       => '__badValue__',
             default       => {},
@@ -2015,7 +2050,7 @@ m{^(?:ldapi://[^/]*/?|\w[\w\-\.]*(?::\d{1,5})?|ldap(?:s|\+tls)?://\w[\w\-\.]*(?:
         authChoiceModules => {
             type       => 'authChoiceContainer',
             keyTest    => qr/^(\d*)?[a-zA-Z0-9_]+$/,
-            keyMsgFail => 'Bad choice key',
+            keyMsgFail => '__badChoiceKey__',
             test       => sub { 1 },
             select     => [
                 [
