@@ -13,7 +13,7 @@
 
 package Lemonldap::NG::Manager::Build::Tree;
 
-our $VERSION = '1.9.0';
+our $VERSION = '1.9.1';
 
 # TODO: Missing:
 #  * activeTimer
@@ -28,7 +28,7 @@ sub tree {
                     title => 'portalParams',
                     help  => 'portal.html',
                     nodes => [
-                        'portal',
+                        '*portal',
                         {
                             title => 'portalMenu',
                             help  => 'portalmenu.html',
@@ -351,7 +351,7 @@ sub tree {
                             ]
                         },
                         {
-                            title => 'webiDParams',
+                            title => 'webidParams',
                             help  => 'authwebid.html',
                             nodes => [
                                 'webIDAuthnLevel', 'webIDExportedVars',
